use std::ops::{Add, AddAssign, Div, DivAssign, Mul, MulAssign, Sub, SubAssign};

#[derive(Debug, Clone, Copy)]
pub struct Cartesian {
	pub x: f64,
	pub y: f64,
}

impl Cartesian {
	pub fn new(x: f64, y: f64) -> Self {
		Cartesian { x, y }
	}

	/*
	pub fn norm(&self) -> f64 {
		return (self.x * self.x + self.y * self.y).sqrt();
	}
	*/
}

impl From<(u32, u32)> for Cartesian {
	fn from(value: (u32, u32)) -> Self {
		Cartesian {
			x: value.0 as f64,
			y: value.1 as f64,
		}
	}
}

impl Add for Cartesian {
	type Output = Self;

	fn add(self, other: Self) -> Self {
		Self {
			x: self.x + other.x,
			y: self.y + other.y,
		}
	}
}

impl AddAssign for Cartesian {
	fn add_assign(&mut self, rhs: Self) {
		self.x += rhs.x;
		self.y += rhs.y;
	}
}

impl Sub for Cartesian {
	type Output = Self;

	fn sub(self, other: Self) -> Self {
		Self {
			x: self.x - other.x,
			y: self.y - other.y,
		}
	}
}

impl SubAssign for Cartesian {
	fn sub_assign(&mut self, rhs: Self) {
		self.x += rhs.x;
		self.y += rhs.y;
	}
}

impl Mul for Cartesian {
	type Output = Self;

	fn mul(self, other: Self) -> Self {
		Self {
			x: self.x * other.x,
			y: self.y * other.y,
		}
	}
}

impl MulAssign for Cartesian {
	fn mul_assign(&mut self, rhs: Self) {
		self.x *= rhs.x;
		self.y *= rhs.y;
	}
}

impl Mul<f64> for Cartesian {
	type Output = Cartesian;
	fn mul(self, rhs: f64) -> Self::Output {
		Self {
			x: self.x * rhs,
			y: self.y * rhs,
		}
	}
}

impl MulAssign<f64> for Cartesian {
	fn mul_assign(&mut self, rhs: f64) {
		self.x *= rhs;
		self.y *= rhs;
	}
}

impl Div<f64> for Cartesian {
	type Output = Cartesian;
	fn div(self, rhs: f64) -> Self::Output {
		Self {
			x: self.x / rhs,
			y: self.y / rhs,
		}
	}
}

impl DivAssign<f64> for Cartesian {
	fn div_assign(&mut self, rhs: f64) {
		self.x /= rhs;
		self.y /= rhs;
	}
}
