use super::Cartesian;
use crate::DrawContext;

#[derive(Debug, Clone, Copy)]

pub struct WorldPosition {
	pub pos: Cartesian, // True world position
	pub par: f64,       // Parallax factor
}

impl WorldPosition {
	pub fn new(pos: Cartesian, par: f64) -> Self {
		WorldPosition { pos: pos, par }
	}

	/*
	fn from_screen_position(dc: &DrawContext, pos: Cartesian, par: f64) -> Self {
		WorldPosition {
			par,
			pos: ((pos * Cartesian::new(1.0, -1.0)) + dc.top_left) * par + dc.camera.pos,
		}
	}
	*/

	/// Transform this world coordinate into a position on the screen,
	/// taking parallax into account. (0, 0) is at top-left corner.
	/// Returned position is this object's center.
	pub fn screen_position(&self, dc: &DrawContext) -> Cartesian {
		let par = self.par;
		let pos: Cartesian = self.pos;
		return (((pos - dc.camera.pos) / par) - dc.top_left) * Cartesian::new(1.0, -1.0);
	}

	/*
	/// Transform this world coordinate into a position on the screen, ignoring parallax.
	/// Used for debugging.
	pub fn screen_position_real(&self, dc: &DrawContext) -> Cartesian {
		let pos: Cartesian = self.pos;
		return ((pos - dc.camera.pos) - dc.top_left) * Cartesian::new(1.0, -1.0);
	}
	*/
}

impl Into<Cartesian> for WorldPosition {
	fn into(self) -> Cartesian {
		self.pos.into()
	}
}
